@@ -1,10 +1,10 @@
 import Adw from 'gi://Adw?version=1';
-import Gdk from 'gi://Gdk?version=4.0';
-import Gio from 'gi://Gio';
 import GLib from 'gi://GLib';
 import GObject from 'gi://GObject';
 import Gtk from 'gi://Gtk?version=4.0';
 
+import {gettext as _} from 'gettext';
+
 import {formatError} from './misc/errorUtils.js';
 
 export class ExtensionPrefsDialog extends Adw.PreferencesWindow {
@@ -68,6 +68,7 @@ export class ExtensionPrefsDialog extends Adw.PreferencesWindow {
         while (this.visible_page)
             this.remove(this.visible_page);
 
+        this.set_title(_('Extension Error'));
         this.add(new ExtensionPrefsErrorPage(this._extension, e));
     }
 }
@@ -78,9 +79,7 @@ class ExtensionPrefsErrorPage extends Adw.PreferencesPage {
         'resource:///org/gnome/Shell/Extensions/ui/extension-error-page.ui';
 
     static [Gtk.internalChildren] = [
-        'expander',
-        'expanderArrow',
-        'revealer',
+        'descriptionLabel',
         'errorView',
     ];
 
@@ -93,47 +92,37 @@ class ExtensionPrefsErrorPage extends Adw.PreferencesPage {
                 const clipboard = self.get_display().get_clipboard();
                 clipboard.set(self._errorMarkdown);
             });
-        this.install_action('page.show-url',
-            null,
-            self => Gtk.show_uri(self.get_root(), self._url, Gdk.CURRENT_TIME));
     }
 
     constructor(extension, error) {
         super();
 
-        this._addCustomStylesheet();
+        const {uuid, name, url} = extension.metadata;
 
-        this._uuid = extension.uuid;
-        this._url = extension.metadata.url || '';
+        let label =
+            /* Translators: %s is an extension name */
+            _('Unable to display the settings for “%s”.')
+                .format(GLib.markup_escape_text(name, -1));
 
-        this.action_set_enabled('page.show-url', this._url !== '');
+        if (url) {
+            /* Translators: Link label in the phrase "Information about this
+               problem may be available on the extension website" */
+            const linkLabel = _('extension website');
 
-        this._gesture = new Gtk.GestureClick({
-            button: 0,
-            exclusive: true,
-        });
-        this._expander.add_controller(this._gesture);
+            label += ' ';
+            /* Translators: %s is "extension website" */
+            label += _('Information about this problem may be available on the %s.')
+                .format(`<a href="${url}">${linkLabel}</a>`);
+        }
 
-        this._gesture.connect('released', (gesture, nPress) => {
-            if (nPress === 1)
-                this._revealer.reveal_child = !this._revealer.reveal_child;
-        });
-
-        this._revealer.connect('notify::reveal-child', () => {
-            this._expanderArrow.icon_name = this._revealer.reveal_child
-                ? 'pan-down-symbolic'
-                : 'pan-end-symbolic';
-            this._syncExpandedStyle();
-        });
-        this._revealer.connect('notify::child-revealed',
-            () => this._syncExpandedStyle());
+        this._descriptionLabel.set({label});
 
         const formattedError = formatError(error);
         this._errorView.buffer.text = formattedError;
 
         // markdown for pasting in gitlab issues
         let lines = [
-            `The settings of extension ${this._uuid} had an error:`,
+            `The settings of extension ${uuid} had an error:`,
             '```',
             formattedError.replace(/\n$/, ''),  // remove trailing newline
             '```',
@@ -141,24 +130,4 @@ class ExtensionPrefsErrorPage extends Adw.PreferencesPage {
         ];
         this._errorMarkdown = lines.join('\n');
     }
-
-    _syncExpandedStyle() {
-        if (this._revealer.reveal_child)
-            this._expander.add_css_class('expanded');
-        else if (!this._revealer.child_revealed)
-            this._expander.remove_css_class('expanded');
-    }
-
-    _addCustomStylesheet() {
-        let provider = new Gtk.CssProvider();
-        let uri = 'resource:///org/gnome/Shell/Extensions/css/application.css';
-        try {
-            provider.load_from_file(Gio.File.new_for_uri(uri));
-        } catch (e) {
-            logError(e, 'Failed to add application style');
-        }
-        Gtk.StyleContext.add_provider_for_display(Gdk.Display.get_default(),
-            provider,
-            Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION);
-    }
 }
