@@ -42,7 +42,7 @@ export function holdKeyboard() {
 
 class KeyboardManager {
     constructor() {
-        // The XKB protocol doesn't allow for more than 4 layouts in a
+        // The XKB protocol doesn't allow for more that 4 layouts in a
         // keymap. Wayland doesn't impose this limit and libxkbcommon can
         // handle up to 32 layouts but since we need to support X clients
         // even as a Wayland compositor, we can't bump this.
@@ -55,7 +55,7 @@ class KeyboardManager {
         this._currentKeymap = null;
     }
 
-    _applyLayoutGroup(group) {
+    async _applyLayoutGroup(group) {
         let options = this._buildOptionsString();
         let [layouts, variants] = this._buildGroupStrings(group);
         let model = this._xkbModel;
@@ -68,11 +68,16 @@ class KeyboardManager {
             return;
 
         this._currentKeymap = {layouts, variants, options, model};
-        global.backend.set_keymap(layouts, variants, options, model);
+        await global.backend.set_keymap_async(layouts, variants, options, model, null);
     }
 
-    _applyLayoutGroupIndex(idx) {
-        global.backend.lock_layout_group(idx);
+    async _applyLayoutGroupIndex(idx) {
+        await global.backend.set_keymap_layout_group_async(idx, null);
+    }
+
+    async _doApply(info) {
+        await this._applyLayoutGroup(info.group);
+        await this._applyLayoutGroupIndex(info.groupIndex);
     }
 
     apply(id) {
@@ -82,10 +87,9 @@ class KeyboardManager {
 
         if (this._current && this._current.group === info.group) {
             if (this._current.groupIndex !== info.groupIndex)
-                this._applyLayoutGroupIndex(info.groupIndex);
+                this._applyLayoutGroupIndex(info.groupIndex).catch(logError);
         } else {
-            this._applyLayoutGroup(info.group);
-            this._applyLayoutGroupIndex(info.groupIndex);
+            this._doApply(info).catch(logError);
         }
 
         this._current = info;
@@ -95,18 +99,17 @@ class KeyboardManager {
         if (!this._current)
             return;
 
-        this._applyLayoutGroup(this._current.group);
-        this._applyLayoutGroupIndex(this._current.groupIndex);
+        this._doApply(this._current).catch(logError);
     }
 
     setUserLayouts(ids) {
         this._current = null;
         this._layoutInfos = {};
 
-        for (const id of ids) {
-            let [found, , , layout, variant] = this._xkbInfo.get_layout_info(id);
+        for (let i = 0; i < ids.length; ++i) {
+            let [found, , , _layout, _variant] = this._xkbInfo.get_layout_info(ids[i]);
             if (found)
-                this._layoutInfos[id] = {id, layout, variant};
+                this._layoutInfos[ids[i]] = {id: ids[i], layout: _layout, variant: _variant};
         }
 
         let i = 0;
@@ -139,10 +142,10 @@ class KeyboardManager {
         if (!found)
             [, , id] = GnomeDesktop.get_input_source_from_locale(DEFAULT_LOCALE);
 
-        let layout, variant;
-        [found, , , layout, variant] = this._xkbInfo.get_layout_info(id);
+        let _layout, _variant;
+        [found, , , _layout, _variant] = this._xkbInfo.get_layout_info(id);
         if (found)
-            return {layout, variant};
+            return {layout: _layout, variant: _variant};
         else
             return {layout: DEFAULT_LAYOUT, variant: DEFAULT_VARIANT};
     }
