@@ -9,7 +9,7 @@ import * as Config from '../misc/config.js';
 import * as ExtensionDownloader from './extensionDownloader.js';
 import {formatError} from '../misc/errorUtils.js';
 import {
-    ExtensionState, ExtensionType, loadExtensionMetadata
+    ExtensionState, ExtensionType, loadExtensionMetadata,
 } from '../misc/extensionUtils.js';
 import * as FileUtils from '../misc/fileUtils.js';
 import * as Main from './main.js';
@@ -656,7 +656,7 @@ export class ExtensionManager extends Signals.EventEmitter {
         if (!this.updatesSupported)
             return;
 
-        for (const {dir, info} of FileUtils.collectFromDatadirs('extension-updates', true)) {
+        for (const {file: dir, info} of FileUtils.collectFromDatadirs('extension-updates', true)) {
             let fileType = info.get_file_type();
             if (fileType !== Gio.FileType.DIRECTORY)
                 continue;
@@ -711,7 +711,7 @@ export class ExtensionManager extends Signals.EventEmitter {
 
         const includeUserDir = global.settings.get_boolean('allow-extension-installation');
         const extensionFiles = [...FileUtils.collectFromDatadirs('extensions', includeUserDir)];
-        const extensionObjects = extensionFiles.map(({dir, info}) => {
+        const extensionObjects = extensionFiles.map(({file: dir, info}) => {
             let fileType = info.get_file_type();
             if (fileType !== Gio.FileType.DIRECTORY)
                 return null;
