@@ -9,7 +9,6 @@ import Shell from 'gi://Shell';
 import St from 'gi://St';
 import * as Signals from '../misc/signals.js';
 
-import * as EdgeDragAction from './edgeDragAction.js';
 import * as InputSourceManager from './status/keyboard.js';
 import * as IBusManager from '../misc/ibusManager.js';
 import * as BoxPointer from './boxpointer.js';
@@ -620,7 +619,11 @@ const EmojiPager = GObject.registerClass({
         const swipeTracker = new SwipeTracker.SwipeTracker(this,
             Clutter.Orientation.HORIZONTAL,
             Shell.ActionMode.NORMAL | Shell.ActionMode.OVERVIEW,
-            {allowDrag: true, allowScroll: true});
+            {
+                allowDrag: true,
+                allowScroll: true,
+                name: 'EmojiPager swipe tracker',
+            });
         swipeTracker.connect('begin', this._onSwipeBegin.bind(this));
         swipeTracker.connect('update', this._onSwipeUpdate.bind(this));
         swipeTracker.connect('end', this._onSwipeEnd.bind(this));
@@ -1072,22 +1075,25 @@ export class KeyboardManager extends Signals.EventEmitter {
             this._syncEnabled();
         });
 
-        const mode = Shell.ActionMode.ALL & ~Shell.ActionMode.LOCK_SCREEN;
-        const bottomDragAction = new EdgeDragAction.EdgeDragAction(St.Side.BOTTOM, mode);
-        bottomDragAction.connect('activated', () => {
-            if (this._keyboard)
-                this._keyboard.gestureActivate(Main.layoutManager.bottomIndex);
+        const allowedModes = Shell.ActionMode.ALL & ~Shell.ActionMode.LOCK_SCREEN;
+        const bottomDragGesture = new Shell.EdgeDragGesture({
+            name: 'OSK show bottom drag',
+            side: St.Side.BOTTOM,
         });
-        bottomDragAction.connect('progress', (_action, progress) => {
-            if (this._keyboard)
-                this._keyboard.gestureProgress(progress);
+        bottomDragGesture.connect('may-recognize', () => {
+            return allowedModes & Main.actionMode;
         });
-        bottomDragAction.connect('gesture-cancel', () => {
-            if (this._keyboard)
-                this._keyboard.gestureCancel();
+        bottomDragGesture.connect('progress', (_action, progress) => {
+            this._keyboard?.gestureProgress(progress);
         });
-        global.stage.add_action_full('osk', Clutter.EventPhase.CAPTURE, bottomDragAction);
-        this._bottomDragAction = bottomDragAction;
+        bottomDragGesture.connect('end', () => {
+            this._keyboard?.gestureActivate(Main.layoutManager.bottomIndex);
+        });
+        bottomDragGesture.connect('cancel', () => {
+            this._keyboard?.gestureCancel();
+        });
+        global.stage.add_action(bottomDragGesture);
+        this._bottomDragGesture = bottomDragGesture;
 
         this._syncEnabled();
     }
@@ -1112,13 +1118,13 @@ export class KeyboardManager extends Signals.EventEmitter {
             this._keyboard = new Keyboard();
             this._keyboard.connect('visibility-changed', () => {
                 this.emit('visibility-changed');
-                this._bottomDragAction.enabled = !this._keyboard.visible;
+                this._bottomDragGesture.enabled = !this._keyboard.visible;
             });
         } else if (!enabled && this._keyboard) {
             this._keyboard.setCursorLocation(null);
             this._keyboard.destroy();
             this._keyboard = null;
-            this._bottomDragAction.enabled = true;
+            this._bottomDragGesture.enabled = true;
         }
     }
 
@@ -1914,21 +1920,11 @@ export const Keyboard = GObject.registerClass({
             ? this._focusWindowStartY - Main.layoutManager.keyboardBox.height
             : this._focusWindowStartY;
 
-        let unmanaged = false;
-        const winUnmanagedId = window.connect('unmanaged', () => {
-            unmanaged = true;
-        });
-
         windowActor.ease({
             y: finalY,
             duration: KEYBOARD_ANIMATION_TIME,
             mode: Clutter.AnimationMode.EASE_OUT_QUAD,
             onStopped: () => {
-                window.disconnect(winUnmanagedId);
-
-                if (unmanaged)
-                    return;
-
                 windowActor.y = finalY;
                 this._windowSlideAnimationComplete(window, finalY);
             },
