@@ -231,7 +231,12 @@ export class Overview extends Signals.EventEmitter {
         const swipeTracker = new SwipeTracker.SwipeTracker(global.stage,
             Clutter.Orientation.VERTICAL,
             Shell.ActionMode.NORMAL | Shell.ActionMode.OVERVIEW,
-            {allowDrag: false, allowScroll: false});
+            {
+                allowDrag: false,
+                allowScroll: false,
+                name: 'Overview swipe tracker',
+                phase: Clutter.EventPhase.CAPTURE,
+            });
         swipeTracker.orientation = Clutter.Orientation.VERTICAL;
         swipeTracker.connect('begin', this._gestureBegin.bind(this));
         swipeTracker.connect('update', this._gestureUpdate.bind(this));
