@@ -51,8 +51,7 @@ class ControlsManagerLayout extends Clutter.LayoutManager {
         this._cachedWorkspaceBoxes = new Map();
         this._postAllocationCallbacks = [];
 
-        stateAdjustment.connectObject('notify::value',
-            () => this.layout_changed(), this);
+        stateAdjustment.connect('notify::value', () => this.layout_changed());
 
         this._workAreaBox = new Clutter.ActorBox();
         global.display.connectObject(
@@ -341,20 +340,19 @@ class ControlsManager extends St.Widget {
         this._workspaceAdjustment = Main.createWorkspacesAdjustment(this);
 
         this._stateAdjustment = new OverviewAdjustment(this);
-        this._stateAdjustment.connectObject('notify::value',
-            () => this._update(), this);
+        this._stateAdjustment.connect('notify::value', this._update.bind(this));
 
         this._searchController = new SearchController.SearchController(
             this._searchEntry,
             this.dash.showAppsButton);
-        this._searchController.connectObject('notify::search-active',
-            () => this._onSearchChanged(), this);
+        this._searchController.connect('notify::search-active', this._onSearchChanged.bind(this));
 
-        Main.layoutManager.connectObject('monitors-changed', () =>
-            this._thumbnailsBox.setMonitorIndex(Main.layoutManager.primaryIndex), this);
+        Main.layoutManager.connect('monitors-changed', () => {
+            this._thumbnailsBox.setMonitorIndex(Main.layoutManager.primaryIndex);
+        });
         this._thumbnailsBox = new WorkspaceThumbnail.ThumbnailsBox(
             this._workspaceAdjustment, Main.layoutManager.primaryIndex);
-        this._thumbnailsBox.connectObject('notify::should-show', () => {
+        this._thumbnailsBox.connect('notify::should-show', () => {
             this._thumbnailsBox.show();
             this._thumbnailsBox.ease_property('expand-fraction',
                 this._thumbnailsBox.should_show ? 1 : 0, {
@@ -362,7 +360,7 @@ class ControlsManager extends St.Widget {
                     mode: Clutter.AnimationMode.EASE_OUT_QUAD,
                     onComplete: () => this._updateThumbnailsBox(),
                 });
-        }, this);
+        });
 
         this._workspacesDisplay = new WorkspacesView.WorkspacesDisplay(
             this,
@@ -386,8 +384,8 @@ class ControlsManager extends St.Widget {
             this.dash,
             this._stateAdjustment);
 
-        this.dash.showAppsButton.connectObject('notify::checked',
-            () => this._onShowAppsButtonToggled(), this);
+        this.dash.showAppsButton.connect('notify::checked',
+            this._onShowAppsButtonToggled.bind(this));
 
         Main.ctrlAltTabManager.addGroup(
             this.appDisplay,
@@ -416,7 +414,7 @@ class ControlsManager extends St.Widget {
         this._a11ySettings = new Gio.Settings({schema_id: A11Y_SCHEMA});
 
         this._lastOverlayKeyTime = 0;
-        global.display.connectObject('overlay-key', () => {
+        global.display.connect('overlay-key', () => {
             if (this._a11ySettings.get_boolean('stickykeys-enable'))
                 return;
 
@@ -435,7 +433,7 @@ class ControlsManager extends St.Widget {
                 this._shiftState(Meta.MotionDirection.UP);
             else
                 Main.overview.toggle();
-        }, this);
+        });
 
         // connect_after to give search controller first dibs on the event
         global.stage.connect_after('key-press-event', (actor, event) => {
@@ -555,7 +553,6 @@ class ControlsManager extends St.Widget {
         if (thumbnailsBoxVisible) {
             this._thumbnailsBox.opacity = 0;
             this._thumbnailsBox.visible = thumbnailsBoxVisible;
-            this._thumbnailsBox.expandFraction = 1.0;
         }
 
         const params = {
@@ -563,9 +560,10 @@ class ControlsManager extends St.Widget {
             duration: animate ? SIDE_CONTROLS_ANIMATION_TIME : 0,
             mode: Clutter.AnimationMode.EASE_OUT_QUAD,
             onComplete: () => {
-                this._thumbnailsBox.visible = thumbnailsBoxVisible;
-                if (!thumbnailsBoxVisible)
-                    this._thumbnailsBox.expandFraction = 0.0;
+                this._thumbnailsBox.set({
+                    visible: thumbnailsBoxVisible,
+                    expandFraction: thumbnailsBoxVisible ? 1.0 : 0.0,
+                });
             },
         };
 
@@ -823,7 +821,7 @@ class ControlsManager extends St.Widget {
         this.ease({
             opacity: 255,
             duration: STARTUP_ANIMATION_TIME,
-            mode: Clutter.AnimationMode.LINEAR,
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
         });
 
         // Search bar falls from the ceiling
