@@ -340,7 +340,7 @@ export class ScreenShield extends Signals.EventEmitter {
 
     _onShortLightbox(lightBox) {
         if (lightBox.active)
-            this._setActive(true);
+            this._completeLockScreenShown();
     }
 
     showDialog() {
@@ -369,22 +369,37 @@ export class ScreenShield extends Signals.EventEmitter {
     }
 
     _showPointer() {
-        this._cursorTracker.set_pointer_visible(true);
+        if (this._cursorVisibleInhibited) {
+            const seat = global.stage.context.get_backend().get_default_seat();
+            this._cursorTracker.uninhibit_cursor_visibility();
+            seat.uninhibit_unfocus();
+            this._cursorVisibleInhibited = false;
+        }
 
         if (this._motionId) {
-            global.stage.disconnect(this._motionId);
+            this._lockDialogGroup.disconnect(this._motionId);
             this._motionId = 0;
         }
     }
 
+    _hidePointer() {
+        if (!this._cursorVisibleInhibited) {
+            const seat = global.stage.context.get_backend().get_default_seat();
+            seat.inhibit_unfocus();
+            this._cursorTracker.inhibit_cursor_visibility();
+            this._cursorVisibleInhibited = true;
+        }
+    }
+
     _hidePointerUntilMotion() {
-        this._motionId = global.stage.connect('captured-event', (stage, event) => {
+        const eventActor = this._lockDialogGroup;
+        this._motionId = eventActor.connect('captured-event', (_, event) => {
             if (event.type() === Clutter.EventType.MOTION)
                 this._showPointer();
 
             return Clutter.EVENT_PROPAGATE;
         });
-        this._cursorTracker.set_pointer_visible(false);
+        this._hidePointer();
     }
 
     _hideLockScreen(animate) {
@@ -499,9 +514,12 @@ export class ScreenShield extends Signals.EventEmitter {
             if (params.fadeToBlack)
                 this._activateFade(this._shortLightbox, 0);
 
-            this._setActive(true);
+            this._completeLockScreenShown();
         }
+    }
 
+    _completeLockScreenShown() {
+        this._setActive(true);
         this.emit('lock-screen-shown');
     }
 
