@@ -16,13 +16,11 @@ import * as InhibitShortcutsDialog from './inhibitShortcutsDialog.js';
 import * as ModalDialog from './modalDialog.js';
 import * as WindowMenu from './windowMenu.js';
 import * as PadOsd from './padOsd.js';
-import * as EdgeDragAction from './edgeDragAction.js';
 import * as CloseDialog from './closeDialog.js';
 import * as SwitchMonitor from './switchMonitor.js';
 import * as IBusManager from '../misc/ibusManager.js';
 import * as WorkspaceAnimation from './workspaceAnimation.js';
 
-import {loadInterfaceXML} from '../misc/fileUtils.js';
 import * as Main from './main.js';
 
 export const SHELL_KEYBINDINGS_SCHEMA = 'org.gnome.shell.keybindings';
@@ -43,12 +41,6 @@ const ONE_SECOND = 1000; // in ms
 
 const MIN_NUM_WORKSPACES = 2;
 
-const GSD_WACOM_BUS_NAME = 'org.gnome.SettingsDaemon.Wacom';
-const GSD_WACOM_OBJECT_PATH = '/org/gnome/SettingsDaemon/Wacom';
-
-const GsdWacomIface = loadInterfaceXML('org.gnome.SettingsDaemon.Wacom');
-const GsdWacomProxy = Gio.DBusProxy.makeProxyWrapper(GsdWacomIface);
-
 const WINDOW_DIMMER_EFFECT_NAME = 'gnome-shell-window-dimmer';
 
 Gio._promisify(Shell, 'util_start_systemd_unit');
@@ -156,7 +148,7 @@ class WindowDimmer extends Clutter.BrightnessContrastEffect {
         });
 
         this.actor.ease_property(`@effects.${this.name}.brightness`, color, {
-            mode: Clutter.AnimationMode.LINEAR,
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
             duration: (dimmed ? DIM_TIME : UNDIM_TIME) * (animate ? 1 : 0),
             onStopped: () => this._syncEnabled(dimmed),
         });
@@ -233,12 +225,12 @@ class WorkspaceTracker {
 
         if (!Meta.prefs_get_dynamic_workspaces()) {
             this._checkWorkspacesId = 0;
-            return GLib.SOURCE_REMOVE;
+            return false;
         }
 
         // Update workspaces only if Dynamic Workspace Management has not been paused by some other function
         if (this._pauseWorkspaceCheck)
-            return GLib.SOURCE_CONTINUE;
+            return true;
 
         for (i = 0; i < this._workspaces.length; i++) {
             let lastRemoved = this._workspaces[i]._lastRemovedWindow;
@@ -298,7 +290,7 @@ class WorkspaceTracker {
         }
 
         this._checkWorkspacesId = 0;
-        return GLib.SOURCE_REMOVE;
+        return false;
     }
 
     keepWorkspaceAlive(workspace, duration) {
@@ -894,27 +886,7 @@ export class WindowManager {
         global.display.connect('show-pad-osd', this._showPadOsd.bind(this));
         global.display.connect('show-osd', (display, monitorIndex, iconName, label) => {
             let icon = Gio.Icon.new_for_string(iconName);
-            Main.osdWindowManager.show(monitorIndex, icon, label, null);
-        });
-
-        this._gsdWacomProxy = new GsdWacomProxy(Gio.DBus.session,
-            GSD_WACOM_BUS_NAME, GSD_WACOM_OBJECT_PATH,
-            (proxy, error) => {
-                if (error)
-                    log(error.message);
-            });
-
-        global.display.connect('pad-mode-switch', (display, pad, _group, _mode) => {
-            let labels = [];
-
-            // FIXME: Fix num buttons
-            for (let i = 0; i < 50; i++) {
-                let str = display.get_pad_action_label(pad, Meta.PadActionType.BUTTON, i);
-                labels.push(str ?? '');
-            }
-
-            this._gsdWacomProxy?.SetOLEDLabelsAsync(
-                pad.get_device_node(), labels).catch(logError);
+            Main.osdWindowManager.showOne(monitorIndex, icon, label);
         });
 
         global.display.connect('init-xserver', (display, task) => {
@@ -938,9 +910,15 @@ export class WindowManager {
         if (Main.sessionMode.hasWorkspaces)
             this._workspaceTracker = new WorkspaceTracker(this);
 
-        let mode = Shell.ActionMode.NORMAL;
-        let topDragAction = new EdgeDragAction.EdgeDragAction(St.Side.TOP, mode);
-        topDragAction.connect('activated',  () => {
+        const allowedModes = Shell.ActionMode.NORMAL;
+        const topDragGesture = new Shell.EdgeDragGesture({
+            name: 'Window unfullscreen top drag',
+            side: St.Side.TOP,
+        });
+        topDragGesture.connect('may-recognize', () => {
+            return allowedModes & Main.actionMode;
+        });
+        topDragGesture.connect('end', () => {
             let currentWindow = global.display.focus_window;
             if (currentWindow)
                 currentWindow.unmake_fullscreen();
@@ -948,14 +926,14 @@ export class WindowManager {
 
         let updateUnfullscreenGesture = () => {
             let currentWindow = global.display.focus_window;
-            topDragAction.enabled = currentWindow && currentWindow.is_fullscreen();
+            topDragGesture.enabled = currentWindow && currentWindow.is_fullscreen();
         };
 
         global.display.connect('notify::focus-window', updateUnfullscreenGesture);
         global.display.connect('in-fullscreen-changed', updateUnfullscreenGesture);
         updateUnfullscreenGesture();
 
-        global.stage.add_action_full('unfullscreen', Clutter.EventPhase.CAPTURE, topDragAction);
+        global.stage.add_action(topDragGesture);
 
         this._workspaceAnimation =
             new WorkspaceAnimation.WorkspaceAnimationController();
@@ -1695,7 +1673,7 @@ export class WindowManager {
         if (!constructor)
             return;
 
-        /* prevent a corner case where both popups show up at once */
+        /* prevent a corner case where both tab and workspace switcher popups show up at once */
         if (this._workspaceSwitcherPopup != null)
             this._workspaceSwitcherPopup.destroy();
 
