@@ -86,15 +86,17 @@ export const WindowClone = GObject.registerClass({
             dragActorMaxSize: Workspace.WINDOW_DND_SIZE,
             dragActorOpacity: Workspace.DRAGGING_WINDOW_OPACITY,
         });
-        this._draggable.connect('drag-begin', this._onDragBegin.bind(this));
-        this._draggable.connect('drag-cancelled', this._onDragCancelled.bind(this));
-        this._draggable.connect('drag-end', this._onDragEnd.bind(this));
+        this._draggable.connectObject(
+            'drag-begin', this._onDragBegin.bind(this),
+            'drag-cancelled', this._onDragCancelled.bind(this),
+            'drag-end', this._onDragEnd.bind(this),
+            this);
         this.inDrag = false;
 
-        const clickAction = new Clutter.ClickAction();
-        clickAction.connect('clicked',
+        const clickGesture = new Clutter.ClickGesture();
+        clickGesture.connect('recognize',
             () => this.emit('selected', Clutter.get_current_event_time()));
-        this._draggable.addClickAction(clickAction);
+        this.add_action(clickGesture);
 
         let iter = win => {
             let actor = win.get_compositor_private();
@@ -629,13 +631,14 @@ export const ThumbnailsBox = GObject.registerClass({
 
         this._thumbnails = [];
 
-        const clickAction = new Clutter.ClickAction();
-        clickAction.connect('clicked', () => {
+        const clickGesture = new Clutter.ClickGesture();
+        clickGesture.connect('recognize', () => {
+            const coords = clickGesture.get_coords();
             this._activateThumbnailAtPoint(
-                ...clickAction.get_coords(),
+                coords.x, coords.y,
                 Clutter.get_current_event_time());
         });
-        this.add_action(clickAction);
+        this.add_action(clickGesture);
 
         Main.overview.connectObject(
             'showing', () => this._createThumbnails(),
@@ -685,7 +688,6 @@ export const ThumbnailsBox = GObject.registerClass({
     _onDestroy() {
         this._destroyThumbnails();
         this._unqueueUpdateStates();
-        this._clearDropPlaceholderLater();
 
         if (this._settings)
             this._settings.run_dispose();
@@ -718,9 +720,7 @@ export const ThumbnailsBox = GObject.registerClass({
         this.queue_relayout();
     }
 
-    _activateThumbnailAtPoint(stageX, stageY, time) {
-        const [r_, x] = this.transform_stage_point(stageX, stageY);
-
+    _activateThumbnailAtPoint(x, y, time) {
         const thumbnail = this._thumbnails.find(t => x >= t.x && x <= t.x + t.width);
         if (thumbnail)
             thumbnail.activate(time);
@@ -769,14 +769,6 @@ export const ThumbnailsBox = GObject.registerClass({
         this.queue_relayout();
     }
 
-    _clearDropPlaceholderLater() {
-        if (!this._dropPlaceholderLater)
-            return;
-
-        global.compositor.get_laters().remove(this._dropPlaceholderLater);
-        delete this._dropPlaceholderLater;
-    }
-
     _getPlaceholderTarget(index, spacing, rtl) {
         const workspace = this._thumbnails[index];
 
@@ -1109,7 +1101,7 @@ export const ThumbnailsBox = GObject.registerClass({
 
             thumbnail.ease_property('slide-position', 1, {
                 duration: SLIDE_ANIMATION_TIME,
-                mode: Clutter.AnimationMode.LINEAR,
+                mode: Clutter.AnimationMode.EASE_OUT_QUAD,
                 onComplete: () => {
                     this._setThumbnailState(thumbnail, ThumbnailState.ANIMATED_OUT);
                     this._queueUpdateStates();
@@ -1187,10 +1179,7 @@ export const ThumbnailsBox = GObject.registerClass({
 
         const laters = global.compositor.get_laters();
         this._updateStateId = laters.add(
-            Meta.LaterType.BEFORE_REDRAW, () => {
-                this._updateStates();
-                return GLib.SOURCE_REMOVE;
-            });
+            Meta.LaterType.BEFORE_REDRAW, () => this._updateStates());
     }
 
     _unqueueUpdateStates() {
@@ -1340,16 +1329,10 @@ export const ThumbnailsBox = GObject.registerClass({
             this._dropPlaceholder.allocate_preferred_size(
                 ...this._dropPlaceholder.get_position());
 
-            this._clearDropPlaceholderLater();
-            if (this._dropPlaceholder.visible) {
-                const laters = global.compositor.get_laters();
-                this._dropPlaceholderLater = laters.add(
-                    Meta.LaterType.BEFORE_REDRAW, () => {
-                        this._dropPlaceholder.hide();
-                        delete this._dropPlaceholderLater;
-                        return GLib.SOURCE_REMOVE;
-                    });
-            }
+            const laters = global.compositor.get_laters();
+            laters.add(Meta.LaterType.BEFORE_REDRAW, () => {
+                this._dropPlaceholder.hide();
+            });
         }
 
         let childBox = new Clutter.ActorBox();
@@ -1377,16 +1360,10 @@ export const ThumbnailsBox = GObject.registerClass({
 
                 this._dropPlaceholder.allocate(childBox);
 
-                this._clearDropPlaceholderLater();
-                if (!this._dropPlaceholder.visible) {
-                    const laters = global.compositor.get_laters();
-                    this._dropPlaceholderLater = laters.add(
-                        Meta.LaterType.BEFORE_REDRAW, () => {
-                            this._dropPlaceholder.show();
-                            delete this._dropPlaceholderLater;
-                            return GLib.SOURCE_REMOVE;
-                        });
-                }
+                const laters = global.compositor.get_laters();
+                laters.add(Meta.LaterType.BEFORE_REDRAW, () => {
+                    this._dropPlaceholder.show();
+                });
                 x += placeholderWidth + spacing;
             }
 
