@@ -840,7 +840,10 @@ class WorkspacesDisplay extends St.Widget {
             Main.layoutManager.overviewGroup,
             Clutter.Orientation.HORIZONTAL,
             Shell.ActionMode.OVERVIEW,
-            {allowDrag: false});
+            {
+                allowDrag: false,
+                name: 'WorkspacesView swipe tracker',
+            });
         this._swipeTracker.allowLongSwipes = true;
         this._swipeTracker.connect('begin', this._switchWorkspaceBegin.bind(this));
         this._swipeTracker.connect('update', this._switchWorkspaceUpdate.bind(this));
