@@ -1,6 +1,5 @@
 /* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
-#ifndef __SHELL_APP_USAGE_H__
-#define __SHELL_APP_USAGE_H__
+#pragma once
 
 #include "shell-app.h"
 #include "shell-window-tracker.h"
@@ -19,5 +18,3 @@ int shell_app_usage_compare (ShellAppUsage *self,
                              const char    *id_b);
 
 G_END_DECLS
-
-#endif /* __SHELL_APP_USAGE_H__ */
