@@ -273,8 +273,7 @@ cr_rgb_to_string (CRRgb const * a_this)
         }
 
         if (str_buf) {
-                result = (guchar *) str_buf->str;
-                g_string_free (str_buf, FALSE);
+                result = (guchar *) g_string_free (str_buf, FALSE);
         }
 
         return result;
