@@ -21,8 +21,7 @@
 #error "Only <st/st.h> can be included directly.h"
 #endif
 
-#ifndef __ST_BIN_H__
-#define __ST_BIN_H__
+#pragma once
 
 #include <st/st-types.h>
 #include <st/st-widget.h>
@@ -49,5 +48,3 @@ void          st_bin_set_child     (StBin        *bin,
 ClutterActor *st_bin_get_child     (StBin        *bin);
 
 G_END_DECLS
-
-#endif /* __ST_BIN_H__ */
