@@ -38,6 +38,7 @@
 #include <glib.h>
 
 #include <clutter/clutter.h>
+#include <clutter/clutter-pango.h>
 
 #include "st-label.h"
 #include "st-private.h"
@@ -428,12 +429,12 @@ st_label_set_text (StLabel     *label,
  * Returns: (transfer none): the #ClutterText used by #StLabel. The actor
  * is owned by the #StLabel and should not be destroyed by the application.
  */
-ClutterActor*
+ClutterText *
 st_label_get_clutter_text (StLabel *label)
 {
   g_return_val_if_fail (ST_LABEL (label), NULL);
 
-  return label->priv->label;
+  return CLUTTER_TEXT (label->priv->label);
 }
 
 
