@@ -98,8 +98,6 @@ enum {
   PROP_0,
 
   PROP_CHILD,
-  PROP_HSCROLL,
-  PROP_VSCROLL,
   PROP_HADJUSTMENT,
   PROP_VADJUSTMENT,
   PROP_HSCROLLBAR_POLICY,
@@ -128,12 +126,6 @@ st_scroll_view_get_property (GObject    *object,
     case PROP_CHILD:
       g_value_set_object (value, priv->child);
       break;
-    case PROP_HSCROLL:
-      g_value_set_object (value, priv->hscroll);
-      break;
-    case PROP_VSCROLL:
-      g_value_set_object (value, priv->vscroll);
-      break;
     case PROP_HADJUSTMENT:
       g_value_set_object (value, priv->hadjustment);
       break;
@@ -869,26 +861,6 @@ st_scroll_view_class_init (StScrollViewClass *klass)
                          ST_TYPE_SCROLLABLE,
                          ST_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY);
 
-  /**
-   * StScrollView:hscroll:
-   *
-   * The horizontal #StScrollBar for the #StScrollView.
-   */
-  props[PROP_HSCROLL] =
-    g_param_spec_object ("hscroll", NULL, NULL,
-                         ST_TYPE_SCROLL_BAR,
-                         ST_PARAM_READABLE | G_PARAM_DEPRECATED);
-
-  /**
-   * StScrollView:vscroll:
-   *
-   * The vertical #StScrollBar for the #StScrollView.
-   */
-  props[PROP_VSCROLL] =
-    g_param_spec_object ("vscroll", NULL, NULL,
-                         ST_TYPE_SCROLL_BAR,
-                         ST_PARAM_READABLE | G_PARAM_DEPRECATED);
-
   /**
    * StScrollView:hadjustment:
    *
@@ -952,7 +924,7 @@ st_scroll_view_class_init (StScrollViewClass *klass)
                           ST_PARAM_READABLE);
 
   /**
-   * StScrollView:enable-mouse-scrolling:
+   * StScrollView:enable-mouse-scrolling: (getter get_mouse_scrolling) (setter set_mouse_scrolling):
    *
    * Whether to enable automatic mouse wheel scrolling.
    */
@@ -1045,7 +1017,7 @@ st_scroll_view_init (StScrollView *self)
                                     NULL);
   scrollbar = g_object_new (ST_TYPE_SCROLL_BAR,
                             "adjustment", priv->hadjustment,
-                            "vertical", FALSE,
+                            "orientation", CLUTTER_ORIENTATION_HORIZONTAL,
                             NULL);
   g_set_weak_pointer (&priv->hscroll, scrollbar);
 
@@ -1054,7 +1026,7 @@ st_scroll_view_init (StScrollView *self)
                                     NULL);
   scrollbar = g_object_new (ST_TYPE_SCROLL_BAR,
                             "adjustment", priv->vadjustment,
-                            "vertical", TRUE,
+                            "orientation", CLUTTER_ORIENTATION_VERTICAL,
                             NULL);
   g_set_weak_pointer (&priv->vscroll, scrollbar);
 
@@ -1138,46 +1110,6 @@ st_scroll_view_set_child (StScrollView *scroll,
   g_object_thaw_notify (G_OBJECT (scroll));
 }
 
-/**
- * st_scroll_view_get_hscroll_bar:
- * @scroll: a #StScrollView
- *
- * Gets the horizontal #StScrollBar of the #StScrollView.
- *
- * Returns: (transfer none): the horizontal scrollbar
- */
-ClutterActor *
-st_scroll_view_get_hscroll_bar (StScrollView *scroll)
-{
-  StScrollViewPrivate *priv;
-
-  g_return_val_if_fail (ST_IS_SCROLL_VIEW (scroll), NULL);
-
-  priv = st_scroll_view_get_instance_private (scroll);
-
-  return priv->hscroll;
-}
-
-/**
- * st_scroll_view_get_vscroll_bar:
- * @scroll: a #StScrollView
- *
- * Gets the vertical scrollbar of the #StScrollView.
- *
- * Returns: (transfer none): the vertical #StScrollBar
- */
-ClutterActor *
-st_scroll_view_get_vscroll_bar (StScrollView *scroll)
-{
-  StScrollViewPrivate *priv;
-
-  g_return_val_if_fail (ST_IS_SCROLL_VIEW (scroll), NULL);
-
-  priv = st_scroll_view_get_instance_private (scroll);
-
-  return priv->vscroll;
-}
-
 /**
  * st_scroll_view_get_hadjustment:
  * @scroll: a #StScrollView
@@ -1495,3 +1427,25 @@ st_scroll_view_get_bar_offsets (StScrollView *scroll,
                                           : 0.;
     }
 }
+
+gboolean
+st_scroll_view_get_hscrollbar_visible (StScrollView *scroll)
+{
+  StScrollViewPrivate *priv;
+
+  g_return_val_if_fail (ST_IS_SCROLL_VIEW (scroll), FALSE);
+
+  priv = st_scroll_view_get_instance_private (scroll);
+  return priv->hscrollbar_visible;
+}
+
+gboolean
+st_scroll_view_get_vscrollbar_visible (StScrollView *scroll)
+{
+  StScrollViewPrivate *priv;
+
+  g_return_val_if_fail (ST_IS_SCROLL_VIEW (scroll), FALSE);
+
+  priv = st_scroll_view_get_instance_private (scroll);
+  return priv->vscrollbar_visible;
+}
